#include <iostream>
#include <map>
#include "headers/Yerbacon.hpp"
#include <future>
#include <filesystem>
using namespace std;

#include "headers/misc.hpp"
#include "headers/arguments.hpp"
#include "headers/transpiler/Target.hpp"
#include "headers/emscripten_compatibility.h"

int main(int argc, char* argv[]) {
    setlocale(LC_ALL, "");
    string target = "lua";
    bool printResult = false,
         parallel = false,
         newLines = true,
         text_provided = false;
    using unit_result = pair<string, optional<Yerbacon::Exception>>;
    using unit = future<unit_result>;
    map<string_view, unit> Units;
    optional<filesystem::path> output_directory;
    #ifdef EMSCRIPTEN
    is_node = MAIN_THREAD_EM_ASM_INT({ return ENVIRONMENT_IS_NODE; });
    #endif
    for (signed int i = 1; i < argc; ++i)
    {
        const string_view currentArgument (argv[i]);
        if (currentArgument == ArgumentShort("printresult")) printResult = true;
        else if (currentArgument == ArgumentAssignable("target")) {
            const string_view value = ArgumentAssignable::getValueFor(currentArgument);
            if (not value.empty()) target = value;
            else Yerbacon::fail("No target was provided.");
        }
        else if (currentArgument == Argument("parallel")) parallel = true;
        else if (currentArgument == ArgumentAssignable("newlines")) {
            const string_view enabled = ArgumentAssignable::getValueFor(currentArgument);
            if (enabled == "off") {
                newLines = false;
            } else if (enabled == "on") {
                newLines = true;
            } else goto invalid_argument;
        }
        else if (currentArgument == ArgumentShort("text")) printResult = text_provided = true;
        else if (currentArgument == ArgumentShort("output"))
            if ((not output_directory.has_value()) and Units.empty() and (i + 1) != argc) {
                mount(argv[i + 1]);
                output_directory = argv[i + 1]; ++i;
            } else goto invalid_argument;
        else if (text_provided || currentArgument.ends_with(".ybcon")) {
            #ifdef _OPENMP
            if (not parallel) omp_set_num_threads(1);
            #endif
            filesystem::path file_path;
            if (not text_provided) file_path = mount(currentArgument);
            Units.insert_or_assign(currentArgument, async(not parallel ? launch::deferred : launch::async, [&, currentArgument, file_path, index = Units.size() + 1]() {
                unit_result resultingPair;
                try {
                    resultingPair.first = Target::forName(target, newLines)->transpileWithTree(
                        parseString(text_provided ? string(currentArgument) + '\n' : getFileContent(file_path.string(/* NOTE: This string is expected to finish with a line ending */)))
                    );
                    if (!text_provided or output_directory.has_value())
                        outputFileContent(
                            (not output_directory.has_value() ? filesystem::path(file_path)
                                : output_directory->append(
                            (not text_provided) ? file_path.filename().string() : to_string(index))
                        ).replace_extension(target).string(), resultingPair.first);
                } catch (const Yerbacon::Exception& error) {
                    resultingPair.first = not text_provided ? file_path.filename().string() : string();
                    resultingPair.second.emplace(error);
                }
                return resultingPair;
            }));
        } else {
            if (argc == 2) {
                if (currentArgument == Argument("version")) {
                    cout << Yerbacon::getVersion();
                } else if (currentArgument == Argument("buildInfo")) {
                    cout << Yerbacon::getBuildInfo();
                } else goto invalid_argument;
                cout << '\n'; exit(EXIT_SUCCESS);
            } else invalid_argument: Yerbacon::fail({"\"", currentArgument.data(), "\" is not a valid argument."});
        }
    }
    if (!Units.empty()) {
        if (printResult) cout << "~~~~[Yerbacon compilation result]~~~~\n\n";
        for (auto entry_iterator = Units.rbegin(); entry_iterator != Units.rend(); ++entry_iterator) {
            const pair result = entry_iterator->second.get();
            const bool is_exception = result.second.has_value();
            if (not is_exception) {
                if (printResult && !(result.first.empty() || all_of(result.first.begin(), result.first.end(), [](const char& character){
                    return isspace(character);
                }))) {
                    cout << result.first << '\n';
                }
            } else {
                cout << "Compilation"; if (not text_provided) cout << " of " << result.first; cout << " has failed with the following error:" << endl;
                cerr << result.second.value().what() << '\n';
                exit(EXIT_FAILURE);
            }
        }
    } else cout << (!text_provided ? "No valid file provided" : "No text was provided") << ".\n";
    exit(EXIT_SUCCESS);
}